package app.views.match;

import app.data.entity.Game;
import app.data.entity.Match;
import app.data.service.MatchService;
import app.data.service.MatchdayService;
import app.data.service.SeasonService;
import app.image.ImageService;
import app.navigation.Navigation;
import app.views.main.MainView;
import app.views.navigation.NavigationViewBase;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.html.Image;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

@CssImport("app/views/match/match-view.css")
@Route(value = "match", layout = MainView.class)
@PageTitle("Schachliga DACH - Results - Matches")
public class MatchView extends NavigationViewBase {

    private final List<Image> images = new ArrayList<>();

    private final ImageService imageService;

    public MatchView(@Autowired SeasonService seasonService,
                     @Autowired MatchdayService matchdayService,
                     @Autowired MatchService matchService,
                     @Autowired ImageService imageService) {
        super(seasonService, matchdayService, matchService, true, true, true);
        this.imageService = imageService;

        this.navigation.setSeasonEnabled(true);
        this.navigation.setMatchdayEnabled(true);
        this.navigation.setMatchEnabled(true);
    }

    @Override
    public String getRoute() {
        return "match";
    }

    @Override
    protected void configureContent() {
        Match match = navigation.getSelectedMatch().orElseThrow();

        removeAll();
        for (Game game : match.getGames()) {
            Image image = new Image(imageService.getVaadinImagePath(game).orElse(""), "image");
            image.setWidth("30%");
            add(image);
        }
    }
}
