package app.views.matchday;

import app.components.navigation.NavigationHeader;
import app.components.navigation.button.NextMatchdayButton;
import app.components.navigation.button.PrevMatchdayButton;
import app.data.bean.CalculatedMatch;
import app.data.entity.Matchday;
import app.data.service.MatchService;
import app.data.service.MatchdayService;
import app.data.service.SeasonService;
import app.navigation.Navigation;
import app.utils.StringUtils;
import app.utils.VaadinUtils;
import app.views.main.MainView;
import app.views.navigation.NavigationViewBase;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.GridVariant;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Optional;

@CssImport("app/views/matchday/matchday-view.css")
@Route(value = "matchday", layout = MainView.class)
@PageTitle("Schachliga DACH - Results - Matchdays")
public class MatchdayView extends NavigationViewBase {

    private final MatchService matchService;

    private final VerticalLayout matchdayLayout = new VerticalLayout();
    private final Label matchdayHeader = new Label();
    private final Grid<CalculatedMatch> grid = new Grid<>();

    private final Button prevButton;
    private final Button nextButton;

    public MatchdayView(@Autowired SeasonService seasonService,
                        @Autowired MatchdayService matchdayService,
                        @Autowired MatchService matchService) {
        super(seasonService, matchdayService, matchService, true, true, false);
        this.matchService = matchService;

        this.navigation.setSeasonEnabled(true);
        this.navigation.setMatchdayEnabled(true);

        prevButton = new PrevMatchdayButton(navigation);
        nextButton = new NextMatchdayButton(navigation);

        addClassName("matchday-view");
        addClassName("content");

        configureContentLayout();
    }

    ////////////
    // LAYOUT //
    ////////////

    private void configureContentLayout() {
        setWidthFull();
        setHeightFull();
        setAlignItems(FlexComponent.Alignment.CENTER);
        add(new NavigationHeader(navigation), matchdayLayout);

        // TODO add background color for content
        matchdayHeader.addClassName("matchday_header"); // TODO: add dates
        matchdayHeader.setWidth("100%");
        HorizontalLayout matchdayHeaderLayout = new HorizontalLayout(prevButton, matchdayHeader, nextButton);
        matchdayHeaderLayout.setWidthFull();

        matchdayLayout.setPadding(false);
        matchdayLayout.add(matchdayHeaderLayout, grid);
        matchdayLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        matchdayLayout.setWidth("");
        matchdayLayout.addClassName("wrapper");

        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer1))
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.END)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.addColumn((ValueProvider<CalculatedMatch, String>) this::getResultString)
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.addColumn(VaadinUtils.getPlayerRenderer(CalculatedMatch::getPlayer2))
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.START)
                .setWidth("13em")
                .setFlexGrow(1);

        grid.setWidth("32em"); // TODO: find a way to set this dynamically based on column widths
        grid.setHeightByRows(true);

        grid.addThemeVariants(GridVariant.LUMO_NO_BORDER,
                GridVariant.LUMO_NO_ROW_BORDERS, GridVariant.LUMO_ROW_STRIPES);

//        grid.addSelectionListener((SelectionListener<Grid<CalculatedMatch>, CalculatedMatch>) selectionEvent -> {
//            Optional<CalculatedMatch> calculatedMatch = selectionEvent.getFirstSelectedItem();
//            calculatedMatch.ifPresent(match -> navigation.selectMatch(match.getMatch()));
//        });
    }

    private String getResultString(CalculatedMatch match) {
        return StringUtils.getResultString(match.getScore1(), match.getScore2());
    }


    /////////////
    // CONTENT //
    /////////////

    @Override
    protected void configureContent() {
        Optional<Matchday> selectedMatchday = navigation.getSelectedMatchday();
        selectedMatchday.ifPresent(matchday -> {
            matchdayHeader.setText(String.format("Matchday %s", matchday.toString()));
            grid.setItems(matchService.getCalculatedMatches(matchday));
        });
    }

    @Override
    public String getRoute() {
        return "matchday";
    }
}
